/*
 * file CompareBaselinesDemo
 * 
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.CreateActivityDemo
 *
 * (C) Copyright IBM Corporation 2010.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.cmapi.samples.cc;

import java.util.List;

import javax.wvcm.Baseline.AddedVersion;
import javax.wvcm.Baseline.ChangedVersion;
import javax.wvcm.Baseline.CompareReport;
import javax.wvcm.Baseline.DeletedVersion;
import javax.wvcm.PropertyRequestItem.PropertyRequest;
import javax.wvcm.Resource;
import javax.wvcm.ResourceList.ResponseIterator;

import com.ibm.rational.wvcm.stp.cc.CcBaseline;
import com.ibm.rational.wvcm.stp.cc.CcBaseline.CompareFlagEx;
import com.ibm.rational.wvcm.stp.cc.CcComponent;
import com.ibm.rational.wvcm.stp.cc.CcStream;
import com.ibm.rational.wvcm.stp.cc.CcVersion;
import com.ibm.rational.wvcm.stp.cc.CcView;

/**
 * Compare two baselines.
 * TODO: This code is incomplete and untested.
 *       Among other things, we need to initialize the view context so that
 *       we can generate view relative paths for the versions.
 */
public class CompareBaselinesDemo extends CcDemoBase {

    public void testCompareBaselines() throws Exception {
        UcmHelper ucmHelper = new UcmHelper(this);

        // Get the integration stream.
        CcStream stream = ucmHelper.getUcmIntegrationStream();
        
        // Fetch the integration stream's foundation baselines and
        // latest baselines.
        PropertyRequest wantedProps = new PropertyRequest(
                CcStream.FOUNDATION_BASELINE_LIST.nest(
                        CcBaseline.COMPONENT),
                CcStream.LATEST_BASELINE_LIST.nest(
                        CcBaseline.COMPONENT));
        stream = (CcStream) stream.doReadProperties(wantedProps);
        List<CcBaseline> foundBls  = stream.getFoundationBaselineList();
        List<CcBaseline> latestBls = stream.getLatestBaselineList();

        assertTrue(foundBls.size() > 0);
        assertTrue(latestBls.size() > 0);

        // Find the latest baseline in the stream that has the same component
        // as the foundation baseline we've chosen.
        CcBaseline foundBl = foundBls.get(0);
        CcComponent comp = foundBl.getComponent();
        CcView viewContext = null; //TODO Create a view on this stream

        for (CcBaseline bl : latestBls) {
            if (bl.getComponent().equals(comp)) {
                compareBaselines(foundBl, bl, viewContext);
                return;
            }
        }

        throw new Exception("Could not find two baselines");
    }

    /**
     * Compare two UCM baselines.
     * Show version differences only - not activity or baseline differences.
     */
    public void
    compareBaselines(CcBaseline bl1, CcBaseline bl2, CcView viewContext)
        throws Exception
    {
        PropertyRequest wantedProps = new PropertyRequest(Resource.DISPLAY_NAME);

        bl1 = (CcBaseline) bl1.doReadProperties(wantedProps);
        bl2 = (CcBaseline) bl2.doReadProperties(wantedProps);

        trace("Comparing baselines:");
        trace("  " + bl1.getDisplayName());
        trace("and");
        trace("  " + bl1.getDisplayName());
        trace("Showing only version diffs (not activity or baseline diffs)");

        /**
         * Compare the two baselines.
         * Only ask for version diffs.
         * Get the version id and the view relative path for each version.
         */
        wantedProps = new PropertyRequest(
                CcVersion.VIEW_RELATIVE_PATH,
                CcVersion.VERSION_NAME);
        CompareFlagEx[] flags = { CompareFlagEx.VERSIONS };
        ResponseIterator<CompareReport> iter =
            bl1.doCompareReportEx(bl2, flags, viewContext, wantedProps);

        while (iter.hasNext()) {

            CompareReport rpt = iter.next();

            if (rpt instanceof AddedVersion) {
                AddedVersion added = (AddedVersion) rpt;
                CcVersion ver = (CcVersion) added.getVersion();
                trace("Added in second baseline: " + ver.getViewRelativePath());

            } else if (rpt instanceof DeletedVersion) {
                DeletedVersion deleted = (DeletedVersion) rpt;
                CcVersion ver = (CcVersion) deleted.getVersion();
                trace("Removed in second baseline: " + ver.getViewRelativePath());
               
            } else if (rpt instanceof ChangedVersion) {
                ChangedVersion changed = (ChangedVersion) rpt;
                CcVersion oldVer = (CcVersion) changed.getOldVersion();
                CcVersion newVer = (CcVersion) changed.getNewVersion();
                trace("Changed between baselines: " + oldVer.getViewRelativePath());
                trace("   " + oldVer.getVersionName() + ", " + newVer.getVersionName());
            } 
        }
    }
}
